"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ts = require("typescript");
var path = require("path");
var fs = require("fs");
var defaultOptions = {
    target: ts.ScriptTarget.Latest,
    module: ts.ModuleKind.CommonJS
};
/**
 * Parses a file with default TS options
 * @param filePath
 */
function parse(filePath) {
    return withCompilerOptions(defaultOptions).parse(filePath);
}
exports.parse = parse;
/**
 * Constructs a parser for a specified tsconfig file.
 */
function withConfig(tsconfigPath) {
    var configJson = JSON.parse(fs.readFileSync(tsconfigPath, 'utf8'));
    var basePath = path.dirname(tsconfigPath);
    var _a = ts.convertCompilerOptionsFromJson(configJson.compilerOptions, basePath, tsconfigPath), options = _a.options, errors = _a.errors;
    if (errors && errors.length) {
        throw errors[0];
    }
    return withCompilerOptions(options);
}
exports.withConfig = withConfig;
/**
 * Constructs a parser for a specified set of TS compiler options.
 */
function withCompilerOptions(compilerOptions) {
    return {
        parse: function (filePath) {
            var program = ts.createProgram([filePath], compilerOptions);
            var parser = new Parser(program);
            var checker = program.getTypeChecker();
            var sourceFile = program.getSourceFile(filePath);
            var moduleSymbol = checker.getSymbolAtLocation(sourceFile);
            var exports = checker.getExportsOfModule(moduleSymbol);
            var components = exports
                .map(function (exp) { return parser.getComponentInfo(exp, sourceFile); })
                .filter(function (comp) { return comp; });
            return components;
        }
    };
}
exports.withCompilerOptions = withCompilerOptions;
var Parser = (function () {
    function Parser(program) {
        this.checker = program.getTypeChecker();
    }
    Parser.prototype.getComponentInfo = function (exp, source) {
        var type = this.checker.getTypeOfSymbolAtLocation(exp, exp.valueDeclaration);
        var propsType = this.extractPropsFromTypeIfStatelessComponent(type);
        if (!propsType) {
            propsType = this.extractPropsFromTypeIfStatefulComponent(type);
        }
        if (propsType) {
            var componentName = computeComponentName(exp, source);
            var props = this.getPropsInfo(propsType);
            return {
                displayName: componentName,
                description: this.findDocComment(exp),
                props: props
            };
        }
        return null;
    };
    Parser.prototype.extractPropsFromTypeIfStatelessComponent = function (type) {
        var callSignatures = type.getCallSignatures();
        if (callSignatures.length) {
            // Could be a stateless component.  Is a function, so the props object we're interested
            // in is the (only) parameter.
            for (var _i = 0, callSignatures_1 = callSignatures; _i < callSignatures_1.length; _i++) {
                var sig = callSignatures_1[_i];
                var params = sig.getParameters();
                if (params.length === 0) {
                    continue;
                }
                // Maybe we could check return type instead, 
                // but not sure if Element, ReactElement<T> are all possible values                
                var propsParam = params[0];
                if (propsParam.name === 'props' || params.length === 1) {
                    return propsParam;
                }
            }
        }
        return null;
    };
    Parser.prototype.extractPropsFromTypeIfStatefulComponent = function (type) {
        var constructSignatures = type.getConstructSignatures();
        if (constructSignatures.length) {
            // React.Component. Is a class, so the props object we're interested
            // in is the type of 'props' property of the object constructed by the class.
            for (var _i = 0, constructSignatures_1 = constructSignatures; _i < constructSignatures_1.length; _i++) {
                var sig = constructSignatures_1[_i];
                var instanceType = sig.getReturnType();
                var props = instanceType.getProperty('props');
                if (props) {
                    return props;
                }
            }
        }
        return null;
    };
    Parser.prototype.getPropsInfo = function (propsObj) {
        var _this = this;
        var propsType = this.checker.getTypeOfSymbolAtLocation(propsObj, propsObj.valueDeclaration);
        var propertiesOfProps = propsType.getProperties();
        var result = {};
        propertiesOfProps.forEach(function (prop) {
            var propName = prop.getName();
            // Find type of prop by looking in context of the props object itself.
            var propType = _this.checker.getTypeOfSymbolAtLocation(prop, propsObj.valueDeclaration);
            var propTypeString = _this.checker.typeToString(propType);
            var isOptional = (prop.getFlags() & ts.SymbolFlags.Optional) !== 0;
            var jsDocComment = _this.findDocComment(prop);
            result[propName] = {
                required: !isOptional,
                type: { name: propTypeString },
                description: jsDocComment,
                // TODO
                defaultValue: null
            };
        });
        return result;
    };
    Parser.prototype.findDocComment = function (symbol) {
        var _this = this;
        var comment = this.getFullJsDocComment(symbol);
        if (comment) {
            return comment;
        }
        var rootSymbols = this.checker.getRootSymbols(symbol);
        var commentsOnRootSymbols = rootSymbols
            .filter(function (x) { return x !== symbol; })
            .map(function (x) { return _this.getFullJsDocComment(x); })
            .filter(function (x) { return !!x; });
        if (commentsOnRootSymbols.length) {
            return commentsOnRootSymbols[0];
        }
        return '';
    };
    /**
     * Extracts a full JsDoc comment from a symbol, even
     * thought TypeScript has broken down the JsDoc comment into plain
     * text and JsDoc tags.
     */
    Parser.prototype.getFullJsDocComment = function (symbol) {
        var mainComment = ts.displayPartsToString(symbol.getDocumentationComment());
        var tags = symbol.getJsDocTags() || [];
        var tagComments = tags.map(function (t) {
            var result = '@' + t.name;
            if (t.text) {
                result += ' ' + t.text;
            }
            return result;
        });
        return (mainComment + '\n' + tagComments.join('\n')).trim();
    };
    return Parser;
}());
function computeComponentName(exp, source) {
    var exportName = exp.getName();
    if (exportName === 'default') {
        // Default export for a file: named after file
        return path.basename(source.fileName, path.extname(source.fileName));
    }
    else {
        return exportName;
    }
}
//# sourceMappingURL=parser.js.map